package TEITokenizer::Controller::Root;

use strict;
use warnings;
use parent 'Catalyst::Controller';

use JSON;
use Text::TEI::Collate::Manuscript qw( tokenize );
use XML::LibXML;

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
#
__PACKAGE__->config->{namespace} = '';

=head1 NAME

TEITokenizer::Controller::Root - Root Controller for TEITokenizer

=head1 DESCRIPTION

[enter your description here]

=head1 METHODS

=cut

=head2 index

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;

    # Hello World
    $c->stash->{template} = 'inputform.tt2';
}

sub form_tokenize :Local {
    my( $self, $c ) = @_;

    my $parser = XML::LibXML->new();
    my $teitext;
    eval {
	$teitext = $parser->parse_string( $c->request->params->{'xmltext'} );
    };
    if( $@ ) {
	$c->stash->{error_msg} = "Could not parse string as XML: $@\n"
	    . "Original string: ''" . $c->request->params->{'xmltext'} . "''\n";
	# TODO save form values
	# TODO don't return HTML in non-interactive mode
	$c->stash->{template} = 'inputform.tt2';
    } else {
	my @wordlist;
	foreach( tokenize( undef, $teitext->documentElement() ) ) {
	    push( @wordlist, { 'token' => $_->comparison_form || '',
			       'punctuation' => $_->punctuation 
				   if $_->punctuation,
			       'accented_form' => $_->accented_form 
				   if $_->accented_form,
			       'placeholders' => $_->placeholders 
				   if $_->placeholders,
		  } );
	}
	if( $c->request->params->{'display'} ) {
	    # Display the text.
	    my @w_strings = map { $_->{token} } @wordlist;
	    $c->stash->{word_tokens} = \@w_strings;
	    $c->stash->{template} = 'result.tt2';
	} else {
	    # Return a JSON stream.
	    $c->response->content_type( 'application/json' );
	    $c->response->content_encoding( 'UTF-8' );
	    $c->response->body( encode_json( \@wordlist ) );
	}
    }
}

sub doc :Local {
    my( $self, $c ) = @_;
    $c->stash->{template} = 'doc.tt2';
}

sub default :Path {
    my ( $self, $c ) = @_;
    $c->response->body( 'Page not found' );
    $c->response->status(404);
}

=head2 end

Attempt to render a view, if needed.

=cut

sub end : ActionClass('RenderView') {}

=head1 AUTHOR

Tara Andrews

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
